%%
% MSWIBRA: 'multiple' zero eigenvalue version of swibra
%
% algorithm:  compute bif-directions to evals mu with abs(mu)<p.nc.mu2 
% store these in p.sol.tau and the first one in p.tau,
% plot the n tangents in figures 21, 22,...,20+n 
% Then use taugen to generate lin.combinations and try cont 
function q=mswibra(dir,fname,varargin)
if(ischar(dir)) p=loadp(dir,fname); 
else p=dir; if nargin>1; varargin{2}=fname; end; fname=inputname(1); 
end
if p.sol.ptype==3; fprintf('\n Hopf-point, use hoswibra\n'); q=[]; return; end
%if(p.sol.ptype~=1 && p.sw.spcont~=1) fprintf('\nInitialization error: not a branch point.\n'); end
u=p.u; lam=getlam(p); tau=p.tau; xi=p.sol.xi; lvar=length(varargin);
pstyle=p.plot.pstyle; del=1e-4; % don't choose del too small!
if lvar>1; p.sol.ds=varargin{2}; gotds=1; lvar=lvar-1; if lvar>1; pstyle=varargin{3}; end
else; p.sol.ds=p.nc.dsmax/10; gotds=0; 
end 
r=resi(p,u); [Gu,Glam]=getder(p,u,r); % residual and jacobians 
% now calc al0, al1, phi0
u0d=tau(1:p.nu+p.nc.nq); al0=tau(p.nu+p.nc.nq+1); 
if al0==0 fprintf('Degenerate branch point with d/ds lambda=0');return; end
m=0; p.sol.muv(1:5)
for i=1:5; if abs(real(p.sol.muv(i)))<p.nc.mu2 && abs(imag(p.sol.muv(i)))<p.nc.mu2; m=m+1;end; end; 
if m==0; fprintf('no zero evals\n'); q=0; return; end
fprintf(['lam=' num2str(lam), ',  lam´=' num2str(al0) ', ' num2str(m),' real zero eigenvalues: ' num2str(p.sol.muv(1:m)) '\n']);
vs=size(Gu,1); opts.v0=ones(vs,1)/vs; opts.disp=0; 
[phi1v,mu]=eigs(Gu,p.mat.M,m,0,opts); muv1=diag(mu); [psi1v,mu]=eigs(Gu',p.mat.M,m,0,opts); muv2=diag(mu);
if norm(muv1-muv2,'inf')/norm(muv1,inf)>1e-6; muv1',muv2', end 
mtau=[];
for j=1:m
  phi1=real(phi1v(:,j));   psi1=psi1v(:,j);psi1=psi1/(phi1'*psi1);
  al1=psi1'*u0d;  phi0=(u0d-al1*phi1)/al0; 
  rp=resi(p,au2u(p,u2au(p,u)+del*phi1)); % residual and jacobian at u+del*phi1 
  [Gup,Glamp]=getder(p,au2u(p,u2au(p,u)+del*phi1),rp); Gud=Gup-Gu; 
  if any(Gud) 
   Glamd=Glamp-Glam; a1=psi1'*(Gud*phi1)/del; 
   b1=psi1'*(Gud*phi0+Glamd)/del; al1b=-(a1*al1/al0+2*b1); 
   fprintf('a1=%g, b1=%g, al1b=%g\n',a1,b1,al1b); 
   if al1b==0 printf('No distinct branch to switch to.');return; end
   tau1=[al1b*phi1+a1*phi0; a1];  
  else; tau1=[phi1; 0]; %fprintf('trivial swibra\n'); % trivial branch 
  end
  tau1=tau1/xinorm(tau1,xi,p.nc.nq,p.sol.xiq);  % HU 
  mtau=[mtau,tau1];
end %j=1:m
p.mat.mtau=mtau; p.tau=mtau(:,1);
bs=p.branch(:,size(p.branch,2)); p=resetc(p); % set counters (note branch set below)
p.sol.ptype=-2; bs(1)=0; bs(2)=p.sol.ptype; % put last of p on new branch
if(~isempty(varargin)); [p,ok]=setfn(p,varargin{1}); if ok~=1; q=p; return; end
else fprintf('warning: problem directory unchanged.\n'); end
p.branch=bs; p.sol.deta=0; p.file.count=0; 
p.fuha.savefu(p); p.file.count=1; q=p;
set(0,'Units','pixels'); scnsize = get(0,'ScreenSize'); % plot 
scnw=scnsize(3); scnh=scnsize(4);sz=scnh/3-scnh*1/20;
u0=p.u; 
for i=1:m
 switch mod(i,3); case 1; pheight=2*scnh/3; case 2; pheight=sz+115; case 0; pheight=0; end; 
 p.u(1:p.nu)=mtau(1:p.nu,i);
 pw=ceil(i/3); pw=pw+1; set(figure(20+i),'Position',[scnw-pw*sz pheight sz sz]);
 plotsol(p,20+i,p.plot.pcmp,pstyle);  xlabel(''); ylabel(''); title(['lam´=' mat2str(mtau(end,i),2)]);
 %set(figure(30+i),'Position',[scnw-2*pw*sz pheight sz sz]);
 %fouplot(p,30+i,1,[4 3]);
end
p.u(1:p.nu)=u0(1:p.nu); fprintf('lam´='); disp(mtau(end,:)); 
