function p=ocinit(p,varargin)
% ocinit: set parameters for comp.of CPs to CSS/CPS 
%
% Varargin is either empty or consists of the four inputs: start-dir,
% start-point, end-dir, end-point in this order.
%
% Parameters:
% p.tomopt: parameters to tweak newtom/mtom
%   maxIt   - maximal number of newton loops per continuation step for
%             newtom
%   err     - maximal accepted absolut error according to max norm for
%             newtom
%   *       - all mtom-options, see tomset and TOM-documentation

%
% p.ocopt: parameters for the whole continuation procedure
%   nti     - number of timesteps if no initial solution is given (and thus
%             will be computed as a linear map from the starting point to the end
%             point)
%   rhoi    - parameterindex of the discount rate rho
%   s0      - Problem structure of the starting point
%   s1      - problem structure of the end point (either CSS or CPS)
%   s1ho    - boolean if s1 is a periodic solution
%   T       - first guess for time length for CP to CSS
%   usec    - secant preditor (usually calculated by NTisc in 2nd step)
%   wT      - time relation between CPS periode and T
%   tv      - vector of time slices
%   start   - boolean if previous calls of NTisc have been made
%   retsw   - retur switch for NTisc - if 0 only final solution, if 1
%             solution for every continuation step
%   msw     - 0: trivial predictor in NTisc, 1: secant predictor in NTisc
%   sig     - actual stepsize for arclength continuation
%   sigmin  - minimal stepsize for arclength continuation
%   sigmax  - maximal stepsize for arclength continuation
%   verb    - verbosity 
%
% p.cp: Structure, which will contain the computed canonical path.
% Empty after initialization.

% load start and end point if not already set
if length(varargin)==4
p.ocopt.s0=loadp(varargin{1},varargin{2});
p.ocopt.s1=loadp(varargin{3},varargin{4});
p.ocopt.fn={varargin{1}, varargin{2}, varargin{3},varargin{4}}; 

elseif nargin>1;  error('Invalid number of inputs, see help.\n');
elseif ~isfield(p.ocopt,'s0');  fprintf('Starting point is not set - p.ocopt.s0 has to be set.\n');
elseif ~isfield(p.ocopt,'s1');  fprintf('Endpoint is not set - p.ocopt.s1 has to be set.\n');
end
if isfield(p.ocopt.s0,'hopf') && isfield(p.ocopt.s0.hopf,'y') % if p is hopf prob struct
    p.ocopt.s0.u(1:p.ocopt.s0.nu)=p.ocopt.s0.hopf.y(:,end); % set reference point x_0 as the endpoint of the circle
end
% check if endpoint is hopf or not to make corresponding computations
if isfield(p.ocopt.s1,'hopf') && isfield(p.ocopt.s1.hopf,'y');  p.ocopt.s1ho=1;
else;  p.ocopt.s1ho=0;
end

% Print parameters and solutions
screenlayout(p.ocopt.s1);
fprintf('parameters at u0: '); disp(mat2str(p.ocopt.s0.u(p.ocopt.s0.nu+1:end)',5)); 
fprintf('parameters at u1: '); disp(mat2str(p.ocopt.s1.u(p.ocopt.s1.nu+1:end)',5));
try
    hoplot(p.ocopt.s0,1,1,p.ocopt.s0.plot.pstyle);
catch
    plotsol(p.ocopt.s0,1,1,p.ocopt.s0.plot.pstyle);
end
title('u0');drawnow;
if p.ocopt.s1ho==1
    try hoplot(p.ocopt.s1,2,1,p.ocopt.s1.plot.pstyle); title('u1'); end
else
    plotsol(p.ocopt.s1,2,1,p.ocopt.s1.plot.pstyle);
end

% set standard pdesolver-parameter
opt=ocstanopt([]); p.tomopt=opt; 
p.tomopt.err=1e-6; % maximal absolut error in newtom
p.tomopt.maxIt=10; % maximal number of iterations in newtom

% some further standard choices
p.ocopt.T=[]; p.ocopt.usec=[];
p.ocopt.wT=1; % time for orbit relative to period of targeted hopf orbit
p.ocopt.tv=[]; % initial t-mesh
p.ocopt.start=1; % first call? (try to find that out in a diff way, p.cp=[] is a first guess if there is no ini-sol given
p.ocopt.retsw=0;  % return-switch for isc*: 0: only final soln, 1: return for all alpha
p.ocopt.msw=0; % predictor in iscnat. 0: trivial, 1: secant
p.ocopt.verb=1; % verbosity switch
p.ocopt.derr=inf; % max (inf-norm) deviation of desired end point to true end point
p.ocopt.mtom=1; 
p.ocopt.sigmin=1e-2; p.ocopt.sigmax=10; % min/max stepsize for arclength

p.cp=[];
